/*
   See pages 176-179 from "Principles of Parallel Programming".

   This version counts iterations but uses deltas for stopping.
*/
#include <stdio.h>
#include <stdlib.h>
#include <omp.h>
#include "2D_SOR.h"
#include "initial_data_1.h"

/*
  Command line args: matrix size, iterations
*/
int main(int argc, char*argv[])
{
   int i, j, iterates;
   double **temp;

   /* read command line arguments */
   if (argc != 3)
   {
      printf("usage: %s n threshold\n", argv[0]);
      return 0;
   }
   n = atoi(argv[1]);           /* number of rows (and columns) */
   threshold = atof(argv[2]);   /* determines when to stop */
   iterates = 0;

   InitializeData(&new, &val, n);

   do
   {
      delta = 0.0;
      /* update each point */
      #pragma omp parallel private(j)
      {
         double  myDelta = 0.0;
         #pragma omp for
         for(i = 1; i < n-1; i++)
         {
            for(j = 1; j < n-1; j++)
            {
               new[i][j] = (val[i-1][j] + val[i][j+1] +
                            val[i+1][j] + val[i][j-1]) / 4;
               myDelta = Max(myDelta, Abs(new[i][j] - val[i][j]));
            }
         }
         #pragma omp critical
         {
            if (myDelta > delta) delta = myDelta;
         }
      }
      iterates++;
      temp = new;   /* prepare for next iteration */
      new = val;
      val = temp;
   }while(delta > threshold);

   WriteData(); /* write the data to a file so that it can be visualized */
   printf("total iterations = %d\n", iterates);
   printf("final delta = %g\n", delta);

   return 0;
}/* main() */
